/*
 * Defines.cpp
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
/*
 * This file defines a bunch of macros and defines that the
 * C-preprocessor uses on .Java files when generating a .java file.
 *
 * See the Makefile for just how that happens.
 *
 * Note: All .Java files are "dependent" on this file (in make terms)
 * so making a change here will cause the world to recompile.
 */

#define JANOS_ROOT_PKG              utah.janos
#define JANOS_CORE_PKG              JANOS_ROOT_PKG##.core
#define JANOS_IO_PKG                JANOS_ROOT_PKG##.io
#define JANOS_LIB_PKG 		    JANOS_ROOT_PKG##.lib
#define JANOS_ROUTE_PKG             JANOS_ROOT_PKG##.route
#define JANOS_TESTS_PKG             JANOS_ROOT_PKG##.tests
#define JANOS_ANTS_ANTS_PKG         JANOS_ROOT_PKG##.ants.ants
#define JANOS_ANTS_UTILS_PKG        JANOS_ROOT_PKG##.ants.utils
#define JANOS_ANTS_ANTS_WRAPPER_PKG JANOS_ANTS_ANTS_PKG##.wrapper

#define JANOS_CORE_PKG_S              "utah.janos.core"
#define JANOS_ANTS_ANTS_PKG_S         "utah.janos.ants.ants"
#define JANOS_ANTS_ANTS_WRAPPER_PKG_S "utah.janos.ants.ants.wrapper"

/*
 * Convenience macros.
 */

#define UNSIGNED_BYTE2INT(x)	((x) & 0xff)

#define UNSIGNED_SHORT2INT(x)	((x) & 0xffff)

#define EXTRACT_SHORT(array, offset) 			\
	(UNSIGNED_SHORT2INT(				\
	 (UNSIGNED_BYTE2INT(array[(offset)]) << 8) | 	\
	 (UNSIGNED_BYTE2INT(array[(int)(offset)+1]))))

#define EXTRACT_INT(array, offset) 				\
	((UNSIGNED_BYTE2INT(array[(offset)]) << 24) | 		\
	 (UNSIGNED_BYTE2INT(array[(offset) + 1]) << 16) | 	\
	 (UNSIGNED_BYTE2INT(array[(offset) + 2]) << 8) | 	\
	 (UNSIGNED_BYTE2INT(array[(offset) + 3])))

/* Make all class state visible for testing */
#ifdef EXPOSED
# define private public
# define PACKAGE public
# define protected public
#endif

/* So we may label our package-private methods with a token */
#ifndef PACKAGE
# define PACKAGE 
#endif

#ifndef DEBUG
# ifndef NDEBUG
# error DEBUG or NDEBUG must be defined on the preprocessor command line
# endif
#endif

#ifdef DEBUG
# define DEBUG_ON true
# define IMPORT_DEBUG import JANOS_CORE_PKG.Debug;
# define IMPORT_TRACE import JANOS_CORE_PKG.Trace;
# define DEBUGONLY(x) x
# define assert(x) Debug.assertCheck(x, #x, __FILE__, __LINE__)
# define assertLocked(x) Debug.checkLocked(x, #x, __FILE__, __LINE__)
# define FATALEXCEPTION(EXEP, MSG) throw Debug.fatalException(EXEP, MSG)
# define panic(STR) throw Debug.dPanic(STR)
# define NOTIMPLEMENTED(MSG) throw Debug.notImplemented(MSG)

/*
# define TRACE(flag, str) {if (Debug.doTrace(Debug.DBG_ ## flag)) { Debug.pTrace(__FILE__, __LINE__, str); }}
# define TRACE(flag, str) { System.err.println(str); }
*/
/* XXX Make Debug a per-flow variable. */
# define TRACE(FLAGS, STR) { if (Trace.check(FLAGS)) { Debug.pTrace(__FILE__, __LINE__, STR); } }
#endif

#ifdef NDEBUG
# define DEBUG_ON false
# define IMPORT_DEBUG 
# define IMPORT_TRACE 
# define DEBUGONLY(x)
# define FATALEXCEPTION(EXEP, MSG) throw new InternalError(EXEP.getMessage() + MSG)
# define panic(str) throw new InternalError(str)
# define NOTIMPLEMENTED(MSG) throw new InternalError(MSG)

# define assert(x)
# define assertLocked(x)
# define TRACE(x, y)
#endif

#ifdef BENCH
# define BENCHONLY(x) x
#else
# define BENCHONLY(x)
#endif
