/*
 * ANEPDemultiplexKey.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 *
 */
package JANOS_ANTS_ANTS_PKG;

import JANOS_CORE_PKG.DemultiplexKey;

/**
 * Match ANEP packets with the specified TypeID.
 * <p>
 * XXX should check version, etc.
 */
PACKAGE class ANEPDemultiplexKey
	extends DemultiplexKey
{
	private final byte byte1;
	private final byte byte2;

	PACKAGE ANEPDemultiplexKey(int additionalLen, short protocolID)
	{
		super(additionalLen + 3);
		this.byte1 = (byte)((protocolID >> 8) & 0xFF);
		this.byte2 = (byte)((protocolID) & 0xFF);
	}

	public int matches(byte[] buf, int packetOffset, int packetLength)
	{
		//     0               *   1           *       2       *           3
		//     0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
		//    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
		//    |   Version     |     Flags     |            Type ID            |
		//    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
		//    |      ANEP Header Length       |     ANEP Packet Length        |
		//    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
		//    ~                            Options                            ~
		//    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
		//    |                               .                               |
		//    ~                            Payload                            ~
		//    |                               .                               |
		//    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

		if (compareByte(buf, packetOffset, packetLength,
				0, Capsule.ANEPVERSION) != 0)
			if (compareByte(buf, packetOffset, packetLength,
					2, byte1) != 0)
				return compareByte(buf, packetOffset, packetLength,
						   3, byte2);
		return 0;
	}

	public String toString()
	{
		return "ANEP Key: [0:="+
			Integer.toHexString(Capsule.ANEPVERSION)+
			"; 2:="+
			Integer.toHexString(byte1)+
			"; 3:="+
			Integer.toHexString(byte2)+
			"]";
	}
}
