/*
 * Booster.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_ANTS_ANTS_PKG;

import JANOS_CORE_PKG.Address;
import JANOS_CORE_PKG.BufferHandle;
import JANOS_CORE_PKG.CommSpaceHandle;
import JANOS_CORE_PKG.Flow;
import JANOS_CORE_PKG.FlowRun;

IMPORT_DEBUG

/**
 * Common functions for AppBooster and ProtocolBooster.
 * <p>
 * This is abstract.  Subclasses must implement Run().
 * <p>
 * XXX changes to this Class impact both AppBooster
 * and ProtocolBooster.
 * <p>
 * The CommSpaceHandle should follow the following convention:
 *   [0] = takeable integer address of this node.
 *   [1] = set of neighbors (XXX this is a hack).
 */
PACKAGE abstract class Booster
	extends FlowRun
{
	private Flow flow;

	protected Node setupLocalNode(Flow flow,
				      CommSpaceHandle commSpaceHandle)
	{
		this.flow = flow; // XXX for newBufferHandle (below).

		// Get the node address.
		int addr = commSpaceHandle.take(0).getInt();

		// The commSpaceHandle zero slot will act as the
		// communication channel for this application
		Node localNode = new Node(
			flow,
			commSpaceHandle.read(1).getCommSpaceHandle(),
			addr,
			// XXX put this in slot 2
			new UnknownMidPackets(
				commSpaceHandle.read(4).getCommSpaceHandle()));

		return localNode;
	}

	protected BufferHandle newBufferHandle()
	{
		return new BufferHandle(flow);
	}

	protected void recycleBufferHandle(BufferHandle bh)
	{
	}
}

