/* 
 *  $Id: ByteArray.Java,v 1.3 1999/09/07 21:15:53 tullmann Exp $
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */
package JANOS_ANTS_ANTS_PKG;

/**
 * Wrapper for an array of Bytes.  Incorporates offset and length.
 * <p>
 * Most of the methods in here are final for performance reasons.
 *
 * @author probably David Weatherall.
 */
public class ByteArray
{
	public byte[] buf;
	public int offset;
	public int len;

	public ByteArray(int len)
	{
		buf = new byte[len];
		offset = 0;
		this.len = len;
	}

	public ByteArray(byte[] buf)
	{
		this.buf = buf;
		this.offset = 0;
		this.len = buf.length;
	}

	public ByteArray(byte[] buf, int offset)
	{
		this(buf, offset, buf.length-offset);
	}

	public ByteArray(byte[] buf, int offset, int len) 
		throws IndexOutOfBoundsException
	{
		// if (offset + len > buf.length || offset < 0 || len < 0)
		//   throw new IndexOutOfBoundsException();
		this.buf = buf;
		this.offset = offset;
		this.len = len;
	}
  
	public ByteArray(ByteArray parent, int offset)
	{
		this(parent, offset, parent.len-offset);
	}

	public ByteArray(ByteArray parent, int offset, int len)
		throws IndexOutOfBoundsException
	{
		// if (offset < 0 || offset + len > parent.len || len < 0)
		// 	 throw new IndexOutOfBoundsException();
		buf = parent.buf;
		this.offset = parent.offset + offset;
		this.len = len;
	}

	public final int length()
	{
		return len;
	}

	public final byte getByte(int index) throws IndexOutOfBoundsException
	{
		// if (index >= len || index < 0) throw new IndexOutOfBoundsException();
		return buf[offset + index];
	}

	public final void setByte(int index, byte val) 
		throws IndexOutOfBoundsException
	{
		// if (index >= len || index < 0) throw new IndexOutOfBoundsException();
		buf[offset+index] = val;
	}

	public final void getBytes(int srcBegin, byte[] dst, int dstBegin, 
				   int length)
	{
		System.arraycopy(buf, offset + srcBegin, dst, dstBegin, length);
	}

	public final void setBytes(int dstBegin, byte[] src, int srcBegin,
				   int length)
	{
		System.arraycopy(src, srcBegin, buf, offset + dstBegin, length);
	}

	public final void setBytes(int dstBegin, ByteArray src, int srcBegin,
				   int length)
	{
		System.arraycopy(src.buf, src.offset + srcBegin, buf, offset + dstBegin, 
				 length);
	}

	/* 
	 * Note: setBytes/getBytes from String have been deleted because they 
	 * used a deprecated API
	 */

	/**
	 * Returns a byte array exactly the right size for
	 * the valid data in the buf.  Note that if this.buf
	 * is the right side it is returned directly, otherwise
	 * a new array is created of the right size.  (Weird.)
	 *
	 * @return array of bytes
	 */
	public final byte[] toBytes()
	{
		if (offset == 0 && len == buf.length)
			return buf;
		else {
			byte[] newbuf = new byte[len];
			System.arraycopy(buf, offset, newbuf, 0, len);
			this.buf = newbuf;
			this.offset = 0;
			return buf;
		}
	}
}
