/* 
 * RouteEntry.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 *  This file is part of Janos.
 *  
 *  Janos is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *  
 *  Janos is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details, in the file
 *  COPYING.  If that file is not present, write to the Free
 *  Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 *  02111-1307, USA
 *  
 *  The University of Utah and the Flux Group requests users to
 *  return to flux-dist@cs.utah.edu any improvements that they make.
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */
package JANOS_ROUTE_PKG;

import JANOS_CORE_PKG.Address;
import JANOS_CORE_PKG.AddressFormatException;
import JANOS_CORE_PKG.NoSuchNodeAddress;
IMPORT_DEBUG

/**
 * A single entry of route table.  A route entry contains both
 * the "logical" address and "physical" address of a Node.
 *
 * @see RouteTable
 * @author David Wetherall
 */
public class RouteEntry {
	// XXX dst, nxt and addr could be final but the
	// constructor gets very hairy if they are.

	private int dst; // logical destination address

	private int nxt; // logical next address
	private Address addr; // physical next address

	// Cost stuff used by the RouteTable update code.
	PACKAGE int cost;
	PACKAGE long expires;
	PACKAGE boolean change;
	
	public RouteEntry(int d, int n, String a)
		throws NumberFormatException, AddressFormatException, NoSuchNodeAddress
	{
		int port;
		String host;

		dst = d;
		nxt = n;
		
		// Parse the physical next hop address
		
		// Split a into host:port
		int sep = a.indexOf(':');
		if (sep == 0)
		{
			host = a;
			port = 4242; // XXXX default port
		}
		else
		{
			try
			{
				host = a.substring(0,sep);
				port = Integer.parseInt(a.substring(sep+1,a.length()));
				// above might throw NumberFormatException
			}
			catch (StringIndexOutOfBoundsException oobe)
			{
				addr = null;
				panic("RouteEntry: parsing problem: " +oobe.getMessage());
			}
		}
		
		addr = AddressFactory.newAddress(nxt, host, port);
		// above might throw NoSuchNodeAddress
	}

	
	public int getLogicalDest()
	{
		return dst;
	}

	public int getLogicalNext()
	{
		return nxt;
	}

	public Address getPhysicalNext()
	{
		return addr;
		
	}
	
	public String toString() {
		return "RouteEntry[" +
			"d=" +Address.toString(dst)+ ";" +
			"n=" +Address.toString(nxt)+ ";" +
			"a=" +addr+
			"]";
	}
}
